#ifndef SEARCH_STATS_H
#define SEARCH_STATS_H 1

#include <iostream>
#include <iomanip>
#include <vector>
#include <map>
#include "RNAParameters.h"




class SearchStats{

 public:
  RNAParameters* RNAParams;

  SearchParameters* SearchParams;


  std::vector<std::vector<std::vector<std::pair<int,int> > > > v_nodes;

  std::vector<std::vector<std::vector<std::pair<int,int> > > > v_personal_best;

  std::vector<std::vector<std::pair<int,int> > >  v_global_best;

  std::vector<std::vector<double> > v_nodes_values;

  std::vector<std::vector<double> > v_personal_best_values;

  std::vector<double> v_global_best_values;

  std::vector<std::vector<std::map<std::pair<int,int>,double> > > v_S1;
	   
  std::vector<std::vector<bool> > v_random_move;

  SearchStats(RNAParameters* RNAParams,SearchParameters* SearchParams){

    this->RNAParams=RNAParams;
    this->SearchParams=SearchParams;
    v_nodes=std::vector<std::vector<std::vector<std::pair<int,int> > > >();
    v_personal_best=std::vector<std::vector<std::vector<std::pair<int,int> > > >();
    v_global_best=std::vector<std::vector<std::pair<int,int> > >();
    v_nodes_values=std::vector<std::vector<double> >();
    v_personal_best_values=std::vector<std::vector<double> >();
    v_global_best_values =std::vector<double>();
    v_S1=std::vector<std::vector<std::map<std::pair<int,int>,double> > >();
    v_random_move=std::vector<std::vector<bool> >();
  }

  void Update(std::vector<std::map<std::pair<int,int>,double> > S1,std::vector<std::vector<std::pair<int,int> > >  nodes,std::vector<std::vector<std::pair<int,int> > >  personal_best,std::vector<std::pair<int,int> > global_best,std::vector<double>nodes_values, std::vector<double> personal_best_values,double global_best_values,std::vector<bool> random_move){
    v_S1.push_back(S1);
    v_nodes.push_back(nodes);
    v_nodes_values.push_back(nodes_values);

    v_personal_best.push_back(personal_best);
    v_personal_best_values.push_back(personal_best_values);

    v_global_best.push_back(global_best);
    v_global_best_values.push_back(global_best_values);

    v_random_move.push_back(random_move);

  }


 std::string Structure(int length, std::vector<std::pair<int,int> > pairs){
   //  std::string Structure(int length, std::vector<std::pair<int,int> > pairs){

    std::string s=std::string(length,'.');
  for(size_t i=0;i<pairs.size();i++) {
    std::pair<int,int> p=pairs[i];
    s[p.first-1]='(';
    s[p.second-1]=')';
  }
  return s;


}


/*  std::vector<double> StructureEnsemble(){ */

/*    std::vector<double> lowest_value=   std::vector<double>(); */

/*  for(int i=0;i<SearchParams->iterations;i++){ */
/*    //std::cout<<"StrukturEnsemble "<<i<<std::endl; */
/*    //std::cout<<RNAParams->RNA<<std::endl; */
/*    std::vector<std::pair<double,std::string> >  ensemble= std::vector<std::pair<double,std::string> >  (); */

/*    //   std::vector<std::string> ensemble=std::vector<std::string>(); */
/*    //std::vector<std::string> ensemble=std::vector<double>(); */
/*   for(int j=0;j<SearchParams->ants;j++){ */
  
/*     std::string s=Structure(RNAParams->RNA.size(),v_nodes[i][j]); */
/*      double energy=energy_of_struct(const_cast<char*>(RNAParams->RNA.c_str()),const_cast<char*>(s.c_str())); */
/*      ensemble.push_back(std::make_pair(energy,s)); */
/*   } */

/*   //xxxx Want a sorted ensemble usually.  */
/*   sort(ensemble.begin(),ensemble.end()); */
/*   // std::cout<<"BEGIN STRUCTURE ENSEMBLE\n"; */
/*   // for(int k=0;k<ensemble.size();k++) std::cout<<ensemble[k].second<<" "<<std::setprecision(4)<<ensemble[k].first<<std::endl; */
/*   // std::cout<<"END STRUCTURE ENSEMBLE\n"; */
/*   lowest_value.push_back(ensemble.front().first); */
/*  } */
/*  // std::cout<<"Set of lowest values for each turn:\n"; */
/*  //for(int i=0;i<lowest_value.size();i++){ */
/*  //  std::cout<<"Turn "+Str(i)+":"+Str(lowest_value[i])+"\n"; */
 
/*  return lowest_value; */
/*  } */


  void GetResults(){
    std::cout<<"global best conformations and their energy values:\n";
    for(size_t i=0;i<v_global_best.size();i++){
      for(size_t j=0;j<v_global_best[i].size();j++){
	std::cout<<RNAParameters::Print(v_global_best[i][j])+" ";
      }
      std::cout<<"energy:"<<v_global_best_values[i]<<"\n";
    }

    std::cout<<"conformations for each particle and their energy values:\n";
    for(int i=0;i<SearchParams->ants;i++){
      std::cout<<"\n"<<"\tconformations for particle "<<i<<"\n";

      //iterate from 0 to iterations+1, because we want all conformations from before the first to after the last move
      for(int j=0;j<SearchParams->iterations+1;j++){
	std::cout<<"\n\tround "<<j<<"\n";
	std::cout<<"\t========"<<"\n";

	std::cout<<"\trandom move: "+Str(v_random_move[j][i])+"\n";
	std::cout<<"\tstochastic vector:";
  for(std::map<std::pair<int,int>,double>::iterator it=v_S1[j][i].begin();it!=v_S1[j][i].end();it++) std::cout<<RNAParameters::Print(it->first)<<" "<<it->second<<"   ";
	std::cout<<"\n";

	std::cout<<"\tconformation:";
	for(size_t k=0;k<v_nodes[j][i].size();k++){
	  std::cout<<RNAParameters::Print(v_nodes[j][i][k])+" ";
	}
	std::cout<<"\n";

        if(j>0){

          //contacts that have been removed 
	  std::vector<std::pair<int,int> > removed=std::vector<std::pair<int,int> >();
	  for(size_t l=0;l<v_nodes[j-1][i].size();l++){
	    if(find(v_nodes[j][i].begin(),v_nodes[j][i].end(),v_nodes[j-1][i][l])==v_nodes[j][i].end()) removed.push_back(v_nodes[j-1][i][l]);
	  }
          if(removed.size()>0){
	    std::cout<<"\tremoved:";
	    std::cout<<RNAParams->Print(removed);
	  }


          //contacts that have been added
	  std::vector<std::pair<int,int> > added=std::vector<std::pair<int,int> >();
	  for(size_t l=0;l<v_nodes[j][i].size();l++){
	    if(find(v_nodes[j-1][i].begin(),v_nodes[j-1][i].end(),v_nodes[j][i][l])==v_nodes[j-1][i].end()) added.push_back(v_nodes[j][i][l]);
	  }
          if(added.size()>0){
	    std::cout<<"\tadded:";
	    std::cout<<RNAParams->Print(added);
	  }
	}

	

        std::cout<<"\tenergy:"<<v_nodes_values[j][i]<<"\n";

        if(j==0 || v_personal_best[j][i]!=v_personal_best[j-1][i]){
	  std::cout<<"\tpersonal best conformation:";
	  for(size_t k=0;k<v_personal_best[j][i].size();k++){
	    std::cout<<RNAParameters::Print(v_personal_best[j][i][k])+" ";
	  }
	  std::cout<<"\n"<<"\tpersonal best energy:"<<v_personal_best_values[j][i]<<"\n";
	  std::cout<<"\n";
	}

  //  cout<<"print S1:\n";
  //  for(map<int,double>::iterator it=S1.begin();it!=S1.end();it++)cout<<Str(it->first)+":"+Str(it->second)+"\n";

      }

    }
  }

};

#endif
