#ifndef SWARM_H
#define SWARM_H 1

//#include "/usr/include/c++/4.0.2/bits/stl_algobase.h"
#include <iostream>
#include <math.h>
#include <cstdlib>
#include <ctime>
#include <limits>
#include <string>
#include <utility>
#include <map>
#include <vector>
#include <algorithm>
#include <iterator>
//#include "vienna.h"
#include "RNAParameters.h"
#include "SearchParameters.h"
#include "SearchStats.h"
#include "utl.h"
#include "options.h"
#include <values.h>
#include <malloc.h>
#include <errno.h>

/* extern "C" { */
/*   #include "findpath.h"   */
/*   #include "fold.h" */
/*   #include "energy_const.h" */
/*   #include "utils.h" */
/* extern float energy_of_struct (char *string, char * struc); */
/* } */

class Swarm {

 public:
 RNAParameters* RNAParams;
  
 SearchParameters* SearchParams;
  
 SearchStats * Stats;
 

 std::vector<std::vector<std::pair<int,int> > > stemlist;

 std::vector<std::map<std::pair<int,int>,double> > SV;

 std::vector<std::map<std::pair<int,int>,double> >S1;
 
 std::vector<std::vector<std::pair<int,int> > > nodes;

 // std::vector<std::vector<int> > nodes;
 
 std::vector<double> fitness;

 std::vector< std::vector<std::pair<int,int> > > pbest;
 // std::vector<std::vector<int> > pbest; 
 
 std::vector<double> pbest_fitness;

 std::vector<std::pair<int,int> > gbest;
 // std::vector<int> gbest;

 std::vector<bool> random_move;

 double gbest_fitness; 


 Swarm(RNAParameters* RNAParams,SearchParameters* SearchParams){
  this->RNAParams=RNAParams;
  this->SearchParams=SearchParams;
  gbest=std::vector<std::pair<int,int> >();
  gbest_fitness=0.0;
  Stats=new SearchStats(RNAParams,SearchParams);
  S1=std::vector<std::map<std::pair<int,int>,double> >(SearchParams->ants);
  random_move=std::vector<bool>();
  for(int i=0;i<SearchParams->ants;i++) random_move.push_back(false);
  nodes=std::vector<std::vector<std::pair<int,int> > >(SearchParams->ants);

 }


 void AddNode(std::vector<std::pair<int,int> >& nodes,std::pair<int,int> node); 


 static bool ConformationHasPair(const std::vector<std::pair<int,int> >& node,std::pair<int,int> p);
 /**
    Umschmelzen
 */


 std::pair<std::vector<std::pair<int,int> >,std::vector<std::pair<int,int> > > ExchangeStacks(int particle);


 /**
    Don't want pairs in Stochastic Vector that cannot be added because they conflict with the current configuration
 
 */


 void CleanUp(int particle);

 std::vector<std::vector<std::pair<int,int> > > ConflictBetweenStacksAndSV(std::vector<std::vector<std::pair<int,int> > >stacks,int particle);

 std::vector<std::pair<int,int> > ConflictBetweenStackAndSV(std::vector<std::pair<int,int> > stack,int particle);



 // std::vector<std::pair<int,int> > IndicesToStack(std::vector<int> indices);

 //std::vector<int> StackToIndices(std::vector<std::pair<int,int> > stack);


 /**
    Given a Stack, can we find an inner loop and the stack that creates it?
 */



 std::vector<std::pair<int,int> > EnvelopStack(int particle);


 std::vector<std::pair<int,int> > EnvelopStack(std::vector<std::pair<int,int> > conformation);



 void GenerateStemList();

 void Initialize();


 bool IsIsolated(const std::vector<std::pair<int,int> >& node,std::pair<int,int> p);

 void Refold(int particle);
 //static std::string Print(int i,int j);

 //static std::string Print(std::pair<int,int> p);

 /**
    Does the Stochastic vector still contain base pairs that can be added?
 */

 bool NoMoreContactsToAdd(int particle);

 //  std::vector<int> CreateStack(int size);


 /**
    Add a newstack that doesn't overlap with previous stacks to the reference set.
 */

  std::vector<std::pair<int,int> > FindStack(int particle,int min_size);

  //std::vector<int> FindStack(std::vector<int> node,int min_size);

  void InitializeNodes();

  void CreateStochasticVector();

  /**
     checks whether base_pair is part of a stack of the conformation node+base_pair 
  */

  // bool InStack(std::vector<int> node,int idx);
  bool InStack(std::vector<std::pair<int,int> > node,std::pair<int,int>  base_pair);


 // std::vector<int> StartNode();


  /**
     Is there a conflict between the basepair and the node.
     If base_pair is already contained in node, it is not called a conflict (even though the base_pair cannot be added, which is what we are usually interested in)
  */


  static bool Conflict(const std::vector<std::pair<int,int> >& node1, const std::vector<std::pair<int,int> >& node2);


  static bool Conflict(const std::vector<std::pair<int,int> >& node, const std::pair<int,int>& p1);
  

  static bool IntroducesPseudoKnot(const std::vector<std::pair<int,int> >& node, const std::pair<int,int>& p1);
     
  static  bool CanUnify(const std::vector<std::pair<int,int> >& node, const std::pair<int,int>& p1);



  void ParticleTurn(int i);


      void Search();

      void MoveParticle(int particle);



 std::vector<std::pair<int,int> > MergeStacks(std::vector<std::pair<int,int> > stack1,std::vector<std::pair<int,int> > stack2);



static  bool StackOrder(const std::vector<std::pair<int,int> >& stack1,const std::vector<std::pair<int,int> >& stack2);

static  bool StackOrder2nd(const std::vector<std::pair<int,int> >& stack1,const std::vector<std::pair<int,int> >& stack2);

 static bool StackOrder2ndInverse(const std::vector<std::pair<int,int> >& stack1,const std::vector<std::pair<int,int> >& stack2);
};





#endif
